//Mixed reality Mirohot with Kinect by Shiou-Min Shen
//KinectTODO.cpp contains functions to process Kinect data stream

//sample reference from j_nyukai
//http://social.msdn.microsoft.com/Forums/en-US/7d303daf-e391-43e2-8bcc-f271d8d40e3e/
//sample reference from King Nguyen
//http://social.msdn.microsoft.com/Forums/en-US/e4f5a696-ed4f-4a5f-8e54-4b3706f62ad0/


#include "kinectTODO.h"

//int ProcessColor(HANDLE h, INuiSensor* sensor, IplImage *cvImageColor)
int ProcessColor(cv::Mat &cvImageColor, HANDLE h, INuiSensor* sensor )
{
	NUI_IMAGE_FRAME colorImageFrame = { 0 };
	INuiFrameTexture* colorFrameTexture;
	NUI_LOCKED_RECT colorLockedRect;      

	HRESULT hr = S_OK;

	hr = sensor->NuiImageStreamGetNextFrame( h, 0, &colorImageFrame );
	if( FAILED( hr ) ){
		std::cerr << "Error : INuiSensor::NuiImageStreamGetNextFrame( Color )" << std::endl;
		return -1;
	}
	colorFrameTexture = colorImageFrame.pFrameTexture;
	hr = colorFrameTexture->LockRect( 0, &colorLockedRect, nullptr, 0 );
	if( FAILED( hr ) ){
		std::cerr << "Error : INuiFrameTexture::LockRect( Color )" << std::endl;
		return -1;
	}
	
	memcpy(cvImageColor.data, colorLockedRect.pBits, colorLockedRect.size);

	// cvShowImage("color_image0", cvImageColor);
	// cvReleaseImage(&image);

	colorFrameTexture->UnlockRect( 0 );
	sensor->NuiImageStreamReleaseFrame( h, &colorImageFrame );
}

//https://msdn.microsoft.com/en-us/library/jj131029.aspx
//int ProcessDepth(HANDLE h, INuiSensor* sensor, IplImage *cvImageDepth, BYTE *depthBuffer, INuiInteractionStream* interactionStream)
int ProcessDepth(cv::Mat &cvImageDepth, BYTE *depthBuffer, HANDLE h, INuiSensor* sensor, INuiInteractionStream* interactionStream )
{
  INuiFrameTexture* depthFrameTexture = 0;
  NUI_IMAGE_FRAME depthImageFrame = { 0 };
  // printf("pass_depth_process\n");
  // Retrieve Depth
  HRESULT hr = S_OK;
  hr = sensor->NuiImageStreamGetNextFrame( h, 0, &depthImageFrame );
  if( FAILED( hr ) )
  {
    std::cerr << "Error : INuiSensor::NuiImageStreamGetNextFrame( Depth )" << std::endl;
    return -1;
  }
  BOOL nearMode = TRUE;
  sensor->NuiImageFrameGetDepthImagePixelFrameTexture( h, &depthImageFrame, &nearMode, &depthFrameTexture );
  NUI_LOCKED_RECT depthLockedRect;
  depthFrameTexture->LockRect( 0, &depthLockedRect, nullptr, 0 );

  if( FAILED( hr ) )
  {
    std::cerr << "Error : INuiFrameTexture::LockRect( Depth )" << std::endl;
    return -1;
  }
  // m_IplImage->SetBufferSize(depthLockedRect.size);

  /*CvSize cv_size;
  cv_size.width  = 1;
  cv_size.height = depthLockedRect.size;*/

  //unsigned char *p_data;
  if(depthLockedRect.Pitch !=0 )
  {     
	  int minDepth = (nearMode ? NUI_IMAGE_DEPTH_MINIMUM_NEAR_MODE : NUI_IMAGE_DEPTH_MINIMUM) >> NUI_IMAGE_PLAYER_INDEX_SHIFT;
        int maxDepth = (nearMode ? NUI_IMAGE_DEPTH_MAXIMUM_NEAR_MODE : NUI_IMAGE_DEPTH_MAXIMUM) >> NUI_IMAGE_PLAYER_INDEX_SHIFT;

        BYTE * rgbrun = depthBuffer;
        const NUI_DEPTH_IMAGE_PIXEL * pBufferRun = reinterpret_cast<const NUI_DEPTH_IMAGE_PIXEL *>(depthLockedRect.pBits);

        // end pixel is start + width*height - 1
        const NUI_DEPTH_IMAGE_PIXEL * pBufferEnd = pBufferRun + (640 * 480);

        while ( pBufferRun < pBufferEnd )
        {
            // discard the portion of the depth that contains only the player index
            USHORT depth = pBufferRun->depth;

            // To convert to a byte, we're discarding the most-significant
            // rather than least-significant bits.
            // We're preserving detail, although the intensity will "wrap."
            // Values outside the reliable depth range are mapped to 0 (black).

            // Note: Using conditionals in this loop could degrade performance.
            // Consider using a lookup table instead when writing production code.
            BYTE intensity = static_cast<BYTE>(depth >= minDepth && depth <= maxDepth ? depth % 256 : 0);

            // Write out blue byte
            *(rgbrun++) = intensity;

            // Write out green byte
            *(rgbrun++) = intensity;

            // Write out red byte
            *(rgbrun++) = intensity;

            // We're outputting BGR, the last byte in the 32 bits is unused so skip it
            // If we were outputting BGRA, we would write alpha here.
            ++rgbrun;

            // Increment our index into the Kinect's depth buffer
            ++pBufferRun;
        }
//process interaction
  hr = interactionStream->ProcessDepth(depthLockedRect.size, PBYTE(depthLockedRect.pBits), depthImageFrame.liTimeStamp);
  // hr = interactionStream->ProcessDepth( depthLockedRect.size, p_data, depthImageFrame.liTimeStamp );
  // free(p_data);
  if( FAILED( hr ) )
  {
	std::cout << "Error : INuiInteractionStream::Process" << std::endl;
	//return -1;
  }  
  depthFrameTexture->UnlockRect( 0 );
  sensor->NuiImageStreamReleaseFrame( h, &depthImageFrame );
}

int ProcessSkeleton(cv::Mat &cvImageColor, CvPoint *SkeletonPts, INuiSensor* sensor, INuiInteractionStream* interactionStream)
{
	HRESULT hr;

	// Retrieve Skeleton
	NUI_SKELETON_FRAME skeletonFrame = { 0 };
	hr = sensor->NuiSkeletonGetNextFrame( 0, &skeletonFrame );
	if( FAILED( hr ) ){
		std::cout << "Error : INuiSensor::NuiSkeletonGetNextFrame" << std::endl;
		return -1;
	}

	sensor->NuiTransformSmooth(&skeletonFrame, NULL);	
	
	for (int i = 0 ; i < NUI_SKELETON_COUNT; ++i)
    {
        NUI_SKELETON_TRACKING_STATE trackingState = skeletonFrame.SkeletonData[i].eTrackingState;

        if (trackingState == NUI_SKELETON_TRACKED )
        {
            // We're tracking the skeleton, draw it
            //DrawSkeleton(skeletonFrame.SkeletonData[i], width, height);			
			DrawSkeleton(cvImageColor, SkeletonPts, sensor, skeletonFrame.SkeletonData[i]);
        }
        else if (trackingState ==  NUI_SKELETON_POSITION_ONLY )
        {
            // we've only received the center point of the skeleton, draw that
			DrawJoint(cvImageColor, skeletonFrame.SkeletonData[i].Position, sensor );
        }
    }			
	// Retrieve Interaction
	Vector4 gravity;// = { 0 };
	sensor->NuiAccelerometerGetCurrentReading( &gravity );
	hr = interactionStream->ProcessSkeleton( NUI_SKELETON_COUNT, skeletonFrame.SkeletonData, &gravity, skeletonFrame.liTimeStamp );
	if( FAILED( hr ) )
	{
	std::cout << "Error : INuiInteractionStream::Process" << std::endl;
		return -1;
	}   	
}


//sample reference from j_nyukai
//http://social.msdn.microsoft.com/Forums/en-US/7d303daf-e391-43e2-8bcc-f271d8d40e3e/

int ProcessInteraction(INuiInteractionStream* interactionStream, UserInfo *iPlayers)
{
	HRESULT hr;
  double scale = 0.0;
  NUI_INTERACTION_FRAME interactionFrame = { 0 };
  hr = interactionStream->GetNextFrame( 0, &interactionFrame );

  FLOAT right_press_extent;

  if( SUCCEEDED( hr ) )
  {
	int iuse = 0;
    for( int user = 0; user <= NUI_SKELETON_COUNT; user++ )
	{
		if (iuse>1)
			return 0;

      NUI_USER_INFO userInfo = interactionFrame.UserInfos[user];

	  NUI_HANDPOINTER_INFO lefthand = userInfo.HandPointerInfos[0];
	  NUI_HANDPOINTER_INFO righthand = userInfo.HandPointerInfos[1];

	  if(userInfo.SkeletonTrackingId > 0 
		  && lefthand.State != NUI_HANDPOINTER_STATE_NOT_TRACKED
		  && lefthand.HandType != NUI_HAND_TYPE_NONE
		  && righthand.State != NUI_HANDPOINTER_STATE_NOT_TRACKED		  
		  && righthand.HandType != NUI_HAND_TYPE_NONE)
	  {		
#if 1		
		  iPlayers[iuse].hands[LEFT].X = lefthand.X;
		  iPlayers[iuse].hands[LEFT].Y = lefthand.Y;
		  iPlayers[iuse].hands[RIGHT].X = righthand.X;
		  iPlayers[iuse].hands[RIGHT].Y = righthand.Y;

		  iPlayers[iuse].hands[LEFT].isValid = true;
		  iPlayers[iuse].hands[RIGHT].isValid = true;
			
		//DWORD right_state = userInfo.HandPointerInfos[1].State;

		switch(lefthand.HandEventType)
		{
			case NUI_HAND_EVENT_TYPE_GRIP:
				//printf("iuse %d, hand 0, X %f, Y %f\n", iuse, iPlayers[iuse].hands[LEFT].X,  iPlayers[iuse].hands[LEFT].Y);
				iPlayers[iuse].hands[LEFT].doGrip = true;
				iPlayers[iuse].hands[LEFT].didGrip = false;
			 //printf("Grip\nuser %d left RawX %lf, RawY %lf\n", user, userInfo.HandPointerInfos[0].RawX,  userInfo.HandPointerInfos[0].RawY);
			 break;
			case NUI_HAND_EVENT_TYPE_GRIPRELEASE:
				//printf("iuse %d, hand 0, X %f, Y %f\n", iuse, iPlayers[iuse].hands[LEFT].X,  iPlayers[iuse].hands[LEFT].Y);
				iPlayers[iuse].hands[LEFT].doGrip = false;
				iPlayers[iuse].hands[LEFT].didGrip = true;
			 //printf("GripRelease\nuser %d left RawX %lf, RawY %lf\n", user, userInfo.HandPointerInfos[0].RawX,  userInfo.HandPointerInfos[0].RawY);
			 break;
			case NUI_HAND_EVENT_TYPE_NONE:
				//printf("iuse %d, hand 0, X %f, Y %f\n", iuse, iPlayers[iuse].hands[LEFT].X,  iPlayers[iuse].hands[LEFT].Y);		  
				//printf("NoEvent\nuser %d left RawX %lf, RawY %lf\n", user, userInfo.HandPointerInfos[0].RawX,  userInfo.HandPointerInfos[0].RawY);
			 break;
			default:
			 break;
		}

		switch(righthand.HandEventType)
		{
			case NUI_HAND_EVENT_TYPE_GRIP:
				//printf("iuse %d, hand 1, X %f, Y %f\n", iuse, iPlayers[iuse].hands[RIGHT].X,  iPlayers[iuse].hands[RIGHT].Y);
				iPlayers[iuse].hands[RIGHT].doGrip = true;
				iPlayers[iuse].hands[RIGHT].didGrip = false;
			 //printf("Grip\nuser %d right RawX %lf, RawY %lf\n", user, userInfo.HandPointerInfos[1].RawX,  userInfo.HandPointerInfos[1].RawY);
			 break;
			case NUI_HAND_EVENT_TYPE_GRIPRELEASE:
				//printf("iuse %d, hand 1, X %f, Y %f\n", iuse, iPlayers[iuse].hands[RIGHT].X,  iPlayers[iuse].hands[RIGHT].Y);
				iPlayers[iuse].hands[RIGHT].doGrip = false;
				iPlayers[iuse].hands[RIGHT].didGrip = true;
			 //printf("GripRelease\nuser %d right RawX %lf, RawY %lf\n", user, userInfo.HandPointerInfos[1].RawX,  userInfo.HandPointerInfos[1].RawY);
			 break;
			case NUI_HAND_EVENT_TYPE_NONE:
				//printf("iuse %d, hand 1, X %f, Y %f\n", iuse, iPlayers[iuse].hands[RIGHT].X,  iPlayers[iuse].hands[RIGHT].Y);
				//printf("NoEvent\nuser %d right RawX %lf, RawY %lf\n", user, userInfo.HandPointerInfos[1].RawX,  userInfo.HandPointerInfos[1].RawY);
			 break;
			default:
			 break;
		}
		iuse +=1;
      }	  
    }
	
  }
  else{
	//iPlayers[0].hands[LEFT].isValid=false;
	//iPlayers[0].hands[RIGHT].isValid=false;

    if( hr == E_POINTER )
	{
      std::cout << "Error : INuiInteractionStream::GetNextFrame( E_POINTER )" << std::endl;
    }
    else if( hr == E_NUI_FRAME_NO_DATA )
	{
      std::cout << "Error : INuiInteractionStream::GetNextFrame( E_NUI_FRAME_NO_DATA )" << std::endl;
    }
  }

  // printf("scale = %d\n", scale);

#endif
  return 0;
}

void DrawJoint(cv::Mat &cvImageColor, Vector4 position, INuiSensor* sensor)
{
	FLOAT depthX = 0.0, depthY = 0.0;
	NuiTransformSkeletonToDepthImage(position, &depthX, &depthY, NUI_IMAGE_RESOLUTION_640x480);
	LONG colorX = 0.0;
	LONG colorY = 0.0;
	sensor->NuiImageGetColorPixelCoordinatesFromDepthPixelAtResolution(NUI_IMAGE_RESOLUTION_640x480, NUI_IMAGE_RESOLUTION_640x480, 0, (LONG)depthX, (LONG)depthY, 0, 	&colorX, &colorY);
	//cvCircle(cvImageColor, cvPoint(colorX, colorY), 8,  CV_RGB(255, 0, 0), -3);
	cv::circle(cvImageColor, cvPoint(colorX, colorY), 8,  CV_RGB(255, 0, 0), -5);
}

//sample reference from King Nguyen
//http://social.msdn.microsoft.com/Forums/en-US/e4f5a696-ed4f-4a5f-8e54-4b3706f62ad0/

/// <summary>
/// Draws a skeleton
/// </summary>
/// <param name="skel">skeleton to draw</param>
/// <param name="windowWidth">width (in pixels) of output buffer</param>
/// <param name="windowHeight">height (in pixels) of output buffer</param>
void DrawSkeleton(cv::Mat &cvImageColor, CvPoint *SkeletonPts, INuiSensor* sensor, const NUI_SKELETON_DATA & skel )
{
	FLOAT depthX;
	FLOAT depthY;
	LONG colorX;
	LONG colorY;

	int i;
	for (i = 0; i < NUI_SKELETON_POSITION_COUNT; ++i)
	{
		depthX = 0.0;
		depthY = 0.0;
		colorX = 0.0;
		colorY = 0.0;

		//SkeletonPts[i] = SkeletonToScreen(skel.SkeletonPositions[i], windowWidth, windowHeight);
	
		NuiTransformSkeletonToDepthImage(skel.SkeletonPositions[i], &depthX, &depthY, NUI_IMAGE_RESOLUTION_640x480);
	
		sensor->NuiImageGetColorPixelCoordinatesFromDepthPixelAtResolution(NUI_IMAGE_RESOLUTION_640x480, NUI_IMAGE_RESOLUTION_640x480, 0, (LONG)depthX, (LONG)depthY, 0, &colorX, &colorY);
	
		SkeletonPts[i] = cvPoint(colorX, colorY);
	}

	// Render Torso
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_HEAD, NUI_SKELETON_POSITION_SHOULDER_CENTER);
	DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_SHOULDER_CENTER, NUI_SKELETON_POSITION_SHOULDER_LEFT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_SHOULDER_CENTER, NUI_SKELETON_POSITION_SHOULDER_RIGHT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_SHOULDER_CENTER, NUI_SKELETON_POSITION_SPINE);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_SPINE, NUI_SKELETON_POSITION_HIP_CENTER);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_HIP_CENTER, NUI_SKELETON_POSITION_HIP_LEFT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_HIP_CENTER, NUI_SKELETON_POSITION_HIP_RIGHT);

    // Left Arm
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_SHOULDER_LEFT, NUI_SKELETON_POSITION_ELBOW_LEFT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_ELBOW_LEFT, NUI_SKELETON_POSITION_WRIST_LEFT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_WRIST_LEFT, NUI_SKELETON_POSITION_HAND_LEFT);

    // Right Arm
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_SHOULDER_RIGHT, NUI_SKELETON_POSITION_ELBOW_RIGHT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_ELBOW_RIGHT, NUI_SKELETON_POSITION_WRIST_RIGHT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_WRIST_RIGHT, NUI_SKELETON_POSITION_HAND_RIGHT);

    // Left Leg
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_HIP_LEFT, NUI_SKELETON_POSITION_KNEE_LEFT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_KNEE_LEFT, NUI_SKELETON_POSITION_ANKLE_LEFT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_ANKLE_LEFT, NUI_SKELETON_POSITION_FOOT_LEFT);

    // Right Leg
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_HIP_RIGHT, NUI_SKELETON_POSITION_KNEE_RIGHT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_KNEE_RIGHT, NUI_SKELETON_POSITION_ANKLE_RIGHT);
    DrawBone(cvImageColor, SkeletonPts, skel, NUI_SKELETON_POSITION_ANKLE_RIGHT, NUI_SKELETON_POSITION_FOOT_RIGHT);
	
	for (i = 0; i < NUI_SKELETON_POSITION_COUNT; ++i)
    {
		//D2D1_ELLIPSE ellipse = D2D1::Ellipse( SkeletonPts[i], g_JointThickness, g_JointThickness );
        if ( skel.eSkeletonPositionTrackingState[i] == NUI_SKELETON_POSITION_INFERRED )
        {	
			cv::circle(cvImageColor, SkeletonPts[i], 8,  CV_RGB(255, 255, 0), -5);
        }
        else if ( skel.eSkeletonPositionTrackingState[i] == NUI_SKELETON_POSITION_TRACKED )
        {
			cv::circle(cvImageColor, SkeletonPts[i], 8,  CV_RGB(255, 64, 0), -5);
        }
    }
}

/// <summary>
/// Draws a bone line between two joints
/// </summary>
/// <param name="skel">skeleton to draw bones from</param>
/// <param name="joint0">joint to start drawing from</param>
/// <param name="joint1">joint to end drawing at</param>
void DrawBone(cv::Mat &cvImageColor, CvPoint *SkeletonPts, const NUI_SKELETON_DATA & skel, NUI_SKELETON_POSITION_INDEX joint0, NUI_SKELETON_POSITION_INDEX joint1)
{
	NUI_SKELETON_POSITION_TRACKING_STATE joint0State = skel.eSkeletonPositionTrackingState[joint0];
    NUI_SKELETON_POSITION_TRACKING_STATE joint1State = skel.eSkeletonPositionTrackingState[joint1];

    // If we can't find either of these joints, exit
    if (joint0State == NUI_SKELETON_POSITION_NOT_TRACKED || joint1State == NUI_SKELETON_POSITION_NOT_TRACKED)
    {
        return;
    }

    // Don't draw if both points are inferred
    if (joint0State == NUI_SKELETON_POSITION_INFERRED && joint1State == NUI_SKELETON_POSITION_INFERRED)
    {
        return;
    }

    // We assume all drawn bones are inferred unless BOTH joints are tracked
    if (joint0State == NUI_SKELETON_POSITION_TRACKED && joint1State == NUI_SKELETON_POSITION_TRACKED)
    {
		cv::line(cvImageColor, SkeletonPts[joint0], SkeletonPts[joint1], CV_RGB(255, 10, 0), 5);
    }
    else
    {
		cv::line(cvImageColor, SkeletonPts[joint0], SkeletonPts[joint1], CV_RGB(0, 128, 255), 5);
    }
}


/// <summary>
/// Converts a skeleton point to screen space
/// </summary>
/// <param name="skeletonPoint">skeleton point to tranform</param>
/// <param name="width">width (in pixels) of output buffer</param>
/// <param name="height">height (in pixels) of output buffer</param>
/// <returns>point in screen-space</returns>
CvPoint2D32f SkeletonToScreen(Vector4 skeletonPoint, int width, int height)
{
    LONG x, y;
    USHORT depth;
	
	static const int        cScreenWidth  = 640;
    static const int        cScreenHeight = 480;

    // Calculate the skeleton's position on the screen
    // NuiTransformSkeletonToDepthImage returns coordinates in NUI_IMAGE_RESOLUTION_320x240 space
    //NuiTransformSkeletonToDepthImage(skeletonPoint, &x, &y, &depth);
	NuiTransformSkeletonToDepthImage(skeletonPoint, &x, &y, &depth, NUI_IMAGE_RESOLUTION_640x480);

    float screenPointX = static_cast<float>(x * width) / cScreenWidth;
    float screenPointY = static_cast<float>(y * height) / cScreenHeight;

    return cvPoint2D32f(screenPointX, screenPointY);
}