//side functions for vector operations and other utilities from before
//written by Shiou-Min Shen

#include "utilitiesGL.h"


vec3::vec3()
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
}


void drawcube(float size)
{
	glBegin(GL_QUADS);                // Begin drawing the color cube with 6 quads
	// Top face (y = 1.0f)
	// Define vertices in counter-clockwise (CCW) order with normal pointing out
	//glColor3f(0.0f, size/2, 0.0f);     // Green
	glVertex3f( size/2, size/2, -size/2);
	glVertex3f(-size/2, size/2, -size/2);
	glVertex3f(-size/2, size/2,  size/2);
	glVertex3f( size/2, size/2,  size/2);

	// Bottom face (y = -size/2)
	//glColor3f(size/2, 0.5f, 0.0f);     // Orange
	glVertex3f( size/2, -size/2,  size/2);
	glVertex3f(-size/2, -size/2,  size/2);
	glVertex3f(-size/2, -size/2, -size/2);
	glVertex3f( size/2, -size/2, -size/2);

	// Front face  (z = size/2)
	//glColor3f(size/2, 0.0f, 0.0f);     // Red
	glVertex3f( size/2,  size/2, size/2);
	glVertex3f(-size/2,  size/2, size/2);
	glVertex3f(-size/2, -size/2, size/2);
	glVertex3f( size/2, -size/2, size/2);

	// Back face (z = -size/2)
	//glColor3f(size/2, size/2, 0.0f);     // Yellow
	glVertex3f( size/2, -size/2, -size/2);
	glVertex3f(-size/2, -size/2, -size/2);
	glVertex3f(-size/2,  size/2, -size/2);
	glVertex3f( size/2,  size/2, -size/2);

	// Left face (x = -size/2)
	//glColor3f(0.0f, 0.0f, size/2);     // Blue
	glVertex3f(-size/2,  size/2,  size/2);
	glVertex3f(-size/2,  size/2, -size/2);
	glVertex3f(-size/2, -size/2, -size/2);
	glVertex3f(-size/2, -size/2,  size/2);

	// Right face (x = size/2)
	//glColor3f(size/2, 0.0f, size/2);     // Magenta
	glVertex3f(size/2,  size/2, -size/2);
	glVertex3f(size/2,  size/2,  size/2);
	glVertex3f(size/2, -size/2,  size/2);
	glVertex3f(size/2, -size/2, -size/2);
	glEnd();  // End of drawing color-cube
}





void HSVtoRGB( GLfloat *r, GLfloat *g, GLfloat *b, GLfloat h, GLfloat s, GLfloat v )
// taken from http://www.cs.rit.edu/~ncs/color/t_convert.html
// r,g,b values are from 0 to 1
// h = [0,360], s = [0,1], v = [0,1]
//		if s == 0, then h = -1 (undefined)
{
	int i;
	GLfloat f, p, q, t;

	if( s == 0 ) {
		// achromatic (grey)
		*r = *g = *b = v;
		return;
	}

	h /= 60;			// sector 0 to 5
	i = floor( h );
	f = h - GLfloat(i);			// factorial part of h
	p = v * ( 1 - s );
	q = v * ( 1 - s * f );
	t = v * ( 1 - s * ( 1 - f ) );

	switch( i ) {
		case 0:
			*r = v;
			*g = t;
			*b = p;
			break;
		case 1:
			*r = q;
			*g = v;
			*b = p;
			break;
		case 2:
			*r = p;
			*g = v;
			*b = t;
			break;
		case 3:
			*r = p;
			*g = q;
			*b = v;
			break;
		case 4:
			*r = t;
			*g = p;
			*b = v;
			break;
		default:		// case 5:
			*r = v;
			*g = p;
			*b = q;
			break;
	}

}


void crossProduct(GLfloat *out, GLfloat *U,GLfloat *V) 
{
	out[0] = U[1] * V[2] - V[1] * U[2];
	out[1] = U[2] * V[0] - V[2] * U[0]; 
	out[2] = U[0] * V[1] - V[0] * U[1];
}

GLfloat dotAngle(GLfloat *U, GLfloat *V )
{
	GLfloat sum = U[0]*V[0] + U[1]*V[1] + U[2]*V[2];
	GLfloat lengthU = sqrt(U[0]*U[0] + U[1]*U[1] + U[2]*U[2]);
	GLfloat lengthV = sqrt(V[0]*V[0] + V[1]*V[1] + V[2]*V[2]);
	
	return acos(GLfloat(sum/lengthU/lengthV));
}

GLfloat dotProduct(GLfloat *U, GLfloat *V )
{
	GLfloat sum = U[0]*V[0] + U[1]*V[1] + U[2]*V[2];
	
	return sum;
}


//useless, gl rotate works way better
void calcRotate( GLfloat angle, GLfloat *axis, GLfloat *rotation )
{	
	GLfloat c = cos(angle);
	GLfloat s = sin(angle);
	GLfloat cComp = 1 - c;
	GLfloat xy = axis[1] * axis[0];
	GLfloat xz = axis[2] * axis[0];
	GLfloat zy = axis[2] * axis[1];
	GLfloat sx = s * axis[0];
	GLfloat sy = s * axis[1];
	GLfloat sz = s * axis[2];
	//float mxxysqr = cComp * axis[1] * axis[1];
	GLfloat mzy = cComp * zy;
	GLfloat mxz = cComp * xz;
	GLfloat mxy = cComp * xy;

	rotation[0] = c + cComp * axis[0] * axis[0];
	rotation[4] = mxy - sz;
	rotation[8] = mxz + sy;

	rotation[1] = mxy + sz;
	rotation[5] = c + cComp * axis[1] * axis[1];
	rotation[9] = mzy - sx;

	rotation[2] = mxz - sy;
	rotation[6] = mzy + sx;
	rotation[10] = c + cComp * axis[2] * axis[2];

        //Apply the transformation matrix to the model's xformation.
	//tMatrix_ = tMatrix_ * tMat;

        //Save the new axis.
	//xAxis_ = tMat * xAxis_; xAxis_.normalize();
	//yAxis_ = tMat * yAxis_; yAxis_.normalize();
	//zAxis_ = tMat * zAxis_; zAxis_.normalize();
}